/**
 *File name ubitx_Kenwood_CAT.cpp
 * CAT Support
 * 
 * This module handles serial communications with external program.
 * Decode and executes Kenwood TS-590S protocol plus
 * Special uBITX remote commands
 * 
 */
#include <Arduino.h>
#include "ubitx.h"
#include "EEPROM.h"

extern void printLine1(char *c);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern void printLine4(char *c);
extern void printLine(char linenmbr, char *c);
extern void updateDisplay(void);
extern int  btnDown(void);
extern void MenuExit(int btn);
extern void setFrequency(unsigned long xf);
extern void stopTx(void);
extern void startTx(byte txMode);
extern void menuSplitToggle( void );
extern void menuVfoToggle(void );
extern void ritDisable(void);
extern void ritEnable(void);
extern void startTx(byte txMode);
extern void stopTx(void);
extern void SetIambicA( void );
extern void SetIambicB( void );
extern void menuPaddleSwapToggle( void );
extern void cwKeydown();
extern void cwKeyUp();
extern void sendmsg(char *str);

extern unsigned long frequency;
extern volatile int ubitx_mode;
extern bool swlUSB;
extern unsigned long vfoA, vfoB;
extern char vfoActive;
extern volatile bool keyDown;
extern unsigned long sideTone;
extern char splitOn;
extern char ritOn;
extern int wpm;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern long ritOffset;
extern char Version_Text[21];
extern char Version_Date[21];
extern bool Volts_Active;
extern volatile unsigned long Ubitx_Voltage;
extern unsigned volatile char IAMBIC;
extern unsigned volatile char PDLSWAP;
extern int wpm;
extern int cwSpeed;
extern char Voltage_Text[7];
unsigned long Volt_Divider;
volatile bool TX_In_Progress = false;

/////////////////////////////////////////////////////////////////

bool Cat_Lock = false;
volatile static byte CAT_BUFF[64];
volatile static int Cat_Buffer_Index;


///////////////////////////////////////////////////////////////////////////////


void Init_Cat( long baud, int portConfig)
{
  Serial.begin(baud,portConfig);
  Serial.flush();
  Cat_Buffer_Index = 0;
}




//////////////////////////////////////////////////////////////////////
void Process_Command( void ){
char Response_Buffer[32];
char Freq_Text[12];
int i,j;
char * temp;
unsigned long Long_Temp;
long rx_freq; 





//////////////////////////////////////////////////////////////    uBITX SPECIFIC CAT COMMANDS  ///////////////////////

  if( strncmp((char*)CAT_BUFF,"uCV;",4) == 0){
    strcpy(Response_Buffer,"uCV");
    if( vfoActive == VFO_A ) strcat(Response_Buffer,"0;");
    else strcat(Response_Buffer,"1;");
    Serial.write(Response_Buffer);
    
  }else if( strncmp((char*)CAT_BUFF,"uLA;",4) == 0){
    ultoa(vfoA, Freq_Text, DEC);
    strcpy(Response_Buffer,"uLA");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
    delay(10);

    
  }else if( strncmp((char*)CAT_BUFF,"uLA",3) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[3],11);
    rx_freq = strtol(Freq_Text,NULL,10);
    if( rx_freq >  400000l)
      if( rx_freq <30000000l){
        vfoA = rx_freq;
        if( vfoActive == VFO_A ){
          frequency = vfoA;
          setFrequency(frequency);
          updateDisplay();
        }
    }
    
  }else if( strncmp((char*)CAT_BUFF,"uLB;",4) == 0){
    ultoa(vfoB, Freq_Text, DEC);
    strcpy(Response_Buffer,"uLB");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
    delay(10);
    
  }else if( strncmp((char*)CAT_BUFF,"uLB",3) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[3],11);
    rx_freq = strtol(Freq_Text,NULL,10);
    if( rx_freq >  400000l)
      if( rx_freq <30000000l){
        vfoB = rx_freq;
        if( vfoActive == VFO_B ){
          frequency = vfoB;
          setFrequency(frequency);
          updateDisplay();
        }
    }
    
  }else if( strncmp((char*)CAT_BUFF,"uDT",3) == 0){
     if( CAT_BUFF[3] == '0') i = 0;
     else if( CAT_BUFF[3] == '1') i = 1;
     else if( CAT_BUFF[3] == '2') i = 2;
     else if( CAT_BUFF[3] == '3') i = 3;
     j = strlen((char*)CAT_BUFF) - 1;
     if( j > 20 )j = 20;
     CAT_BUFF[j]=0;
     if( i > 3 ) i = 3;
     strcpy(Response_Buffer,(char*)&CAT_BUFF[4]);   
     printLine(i,Response_Buffer);
    
  }else if( strncmp((char*)CAT_BUFF,"uFI;",4) == 0){
    strcpy(Response_Buffer,"uFI");
    itoa(Freq_Index,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
      
  }else if( strncmp((char*)CAT_BUFF,"uFI",3) == 0){
    Freq_Index = atoi((char*) &CAT_BUFF[3]);
    if( Freq_Index > 5 ) Freq_Index = 5;
    if( Freq_Index < 0 ) Freq_Index = 0;
    updateDisplay();
      
  }else if( strncmp((char*)CAT_BUFF,"uFL",3) == 0){
    Cat_Lock = atoi((char*) &CAT_BUFF[3]);
    if( Cat_Lock == 0 ){
      EEPROM.put(VFO_A, vfoA);
      EEPROM.put(VFO_B, vfoB);
    }else{
      strcpy(Response_Buffer,"uFL;");
      Serial.write(Response_Buffer);
      delay(10);
    }
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"uKIA;",5) == 0){
    SetIambicA();
    updateDisplay();
     
  }else if( strncmp((char*)CAT_BUFF,"uKIB;",5) == 0){
    SetIambicB();
    updateDisplay();
     
  }else if( strncmp((char*)CAT_BUFF,"uKPN;",5) == 0){
    PDLSWAP = 0x08;   
    menuPaddleSwapToggle();
    updateDisplay();
         
  }else if( strncmp((char*)CAT_BUFF,"uKPR;",5) == 0){
    PDLSWAP = 0;   
    menuPaddleSwapToggle();
    updateDisplay();
        
  }else if( strncmp((char*)CAT_BUFF,"uKS;",4) == 0){
    strcpy(Response_Buffer,"uKS");
    itoa(wpm,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
  
  }else if( strncmp((char*)CAT_BUFF,"uKS",3) == 0){
    wpm = strtol((char*)&CAT_BUFF[3], &temp, 10);
    cwSpeed = 600 / wpm;
    EEPROM.put(CW_SPEED, wpm);
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"uMD;",4) == 0){
    strcpy(Response_Buffer,"uMD");
    if( ubitx_mode == MODE_USB )strcat(Response_Buffer,"0");
    else if( ubitx_mode == MODE_LSB )strcat(Response_Buffer,"1");
    else if( ubitx_mode == MODE_CW )strcat(Response_Buffer,"2");
    else if( ubitx_mode == MODE_CWR )strcat(Response_Buffer,"3");
    else if( ubitx_mode == MODE_SWU )strcat(Response_Buffer,"4");
    else if( ubitx_mode == MODE_SWL )strcat(Response_Buffer,"5");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);  

  }else if( strncmp((char*)CAT_BUFF,"uMD",3) == 0){
    if( CAT_BUFF[3] == '0' )ubitx_mode = MODE_USB;
    else if( CAT_BUFF[3] == '1' ) ubitx_mode = MODE_LSB;
    else if( CAT_BUFF[3] == '2' ) ubitx_mode = MODE_CW;
    else if( CAT_BUFF[3] == '3' ) ubitx_mode = MODE_CWR;
    else if( CAT_BUFF[3] == '4' ) ubitx_mode = MODE_SWU;
    else if( CAT_BUFF[3] == '5' ) ubitx_mode = MODE_SWL;
    i = ubitx_mode;
    EEPROM.put(MODE, i );
    delay(200);
    updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"uRE;",4) == 0){
    ritOn = false;
    ritDisable();
    setFrequency(frequency);
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"uRS;",4) == 0){
    ritOn = true;
    ritEnable();
    setFrequency(frequency);
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"uRO",3) == 0){
    i = atol((char*)&CAT_BUFF[3]);
    if( i <=5000 && i>= -5000 ){
      ritOffset = i;
      setFrequency(frequency);
      updateDisplay();
    }

  }else if( strncmp((char*)CAT_BUFF,"uSA;",4) == 0){
      vfoActive = VFO_A;
      frequency = vfoA;
      ritDisable();
      setFrequency(frequency);
      updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"uSB;",4) == 0){
        vfoActive = VFO_B;
        frequency = vfoB;
      ritDisable();
      setFrequency(frequency);
      updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"uSC;",4) == 0){
    strcpy(Response_Buffer,"uSC");
    for (i=0;i<=40;i=i+4){
      EEPROM.get(i,Long_Temp);
      sprintf(Response_Buffer,"uSC%02d%08x;",i,Long_Temp);
      Serial.write(Response_Buffer);
    }
    for (i=46;i<=62;i=i+4){
      EEPROM.get(i,Long_Temp);
      sprintf(Response_Buffer,"uSC%02d%08x;",i,Long_Temp);
      Serial.write(Response_Buffer);
    }

  }else if( strncmp((char*)CAT_BUFF,"uSE;",4) == 0){
    splitOn = false;
    updateDisplay(); 
    
  }else if( strncmp((char*)CAT_BUFF,"uSS;",4) == 0){
    splitOn = true;
    vfoB = vfoA;
    updateDisplay(); 
   
  }else if( strncmp((char*)CAT_BUFF,"uKT;",4) == 0){
    strcpy(Response_Buffer,"uKT");
    itoa(sideTone,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"uKT",3) == 0){
    sideTone = strtol((char*)&CAT_BUFF[3], &temp, 10);
    //save the setting
    EEPROM.put(CW_SIDETONE, sideTone);
    updateDisplay();
 
  }else if( strncmp((char*)CAT_BUFF,"uVE;",4) == 0){
    Volts_Active = false;
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"uVS;",4) == 0){
    Volts_Active = true;
    updateDisplay();
  
  }else if( strncmp((char*)CAT_BUFF,"uVR;",4) == 0){
    for(i=0;i<32;i++)Response_Buffer[i] = 0;
    sprintf(Response_Buffer,"uVR%s;",Voltage_Text);
    Response_Buffer[strlen(Response_Buffer)] = 0;
    Serial.write(Response_Buffer);
    delay(10);

  }else if( strncmp((char*)CAT_BUFF,"uSM",3) == 0){
    sprintf(Response_Buffer,"%s",&CAT_BUFF[3]);
    Response_Buffer[strlen(Response_Buffer)-1] = 0;
    sendmsg(Response_Buffer);
    strcpy(Response_Buffer,"uSM;");
    Serial.write(Response_Buffer);
    delay(10);

  }else if( strncmp((char*)CAT_BUFF,"uXS;",4) == 0){
    TX_In_Progress = true;
    startTx(TX_SSB);
    
  }else if( strncmp((char*)CAT_BUFF,"uXE;",4) == 0){
    TX_In_Progress = false;
    stopTx();

  }else if( strncmp((char*)CAT_BUFF,"uTS;",4) == 0){
    TX_In_Progress = true;
    startTx(TX_CW);
    cwKeydown();
    
  }else if( strncmp((char*)CAT_BUFF,"uTE;",4) == 0){
    cwKeyUp();
    TX_In_Progress = false;
    stopTx();
  }



  else if( strncmp((char*)CAT_BUFF,"FA;",3) == 0){
    ultoa(vfoA, Freq_Text, DEC);
    strcpy(Response_Buffer,"FA");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FA",2) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[2],11);
    vfoA = strtol(Freq_Text,NULL,10);
    frequency = strtol(Freq_Text,NULL,10);
    setFrequency(frequency);
    updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"FB;",3) == 0){
    ultoa(vfoB, Freq_Text, DEC);
    strcpy(Response_Buffer,"FB");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
    

  }else if( strncmp((char*)CAT_BUFF,"FB",2) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[2],11);
    vfoB= strtol(Freq_Text,NULL,10);
    frequency = strtol(Freq_Text,NULL,10);
    setFrequency(frequency);
    updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"FR;",3) == 0){
    strcpy(Response_Buffer,"FR");
    if( vfoActive == VFO_A )strcat(Response_Buffer,"0");
    else  strcat(Response_Buffer,"1");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FR",2) == 0){
    if( CAT_BUFF[2] == '0' ) vfoActive = VFO_A;
    else vfoActive = VFO_B;
    menuVfoToggle();
    
  }else if( strncmp((char*)CAT_BUFF,"FT;",3) == 0){
    strcpy(Response_Buffer,"FT");
    if( splitOn )strcat(Response_Buffer,"1");
    else  strcat(Response_Buffer,"0");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FT",2) == 0){
    if( CAT_BUFF[2] == '0' ) splitOn = true;
    else splitOn = false;
    menuSplitToggle();
    
  }else if( strncmp((char*)CAT_BUFF,"KS;",3) == 0){
    strcpy(Response_Buffer,"KS");
    itoa(wpm,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"KS",2) == 0){
    wpm = strtol((char*)&CAT_BUFF[2], &temp, 10);
    updateDisplay();
    
    
  }else if( strncmp((char*)CAT_BUFF,"MD;",3) == 0){
    strcpy(Response_Buffer,"MD");
    if( ubitx_mode == MODE_CW )strcat(Response_Buffer,"3");
    else if( ubitx_mode == MODE_USB )strcat(Response_Buffer,"2");
    else if( ubitx_mode == MODE_LSB )strcat(Response_Buffer,"1");
    else strcat(Response_Buffer,"7");
    strcat(Response_Buffer,";");
    Serial.println(Response_Buffer);  
    
  }else if( strncmp((char*)CAT_BUFF,"MD",2) == 0){
    if( CAT_BUFF[2] == '1' )ubitx_mode = MODE_USB;
    else if( CAT_BUFF[2] == '2' ) ubitx_mode = MODE_LSB;
    else if( CAT_BUFF[2] == '3' ) ubitx_mode = MODE_CW;
    else if( CAT_BUFF[2] == '7' ) ubitx_mode = MODE_CWR;
    i = ubitx_mode;
    EEPROM.put(MODE, i );
    setFrequency(frequency);
    updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"RT;",3) == 0){
    strcpy(Response_Buffer,"RT");
    if( ritOn )strcat(Response_Buffer,"1");
    else  strcat(Response_Buffer,"0");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
   
  }else if( strncmp((char*)CAT_BUFF,"RT",2) == 0){
    if( CAT_BUFF[2] == '0' ) ritDisable();
    else ritEnable();
    updateDisplay();
    
   
  }else if( strncmp((char*)CAT_BUFF,"RX;",3) == 0){
    TX_In_Progress = false;
    stopTx();
        
  }else if( strncmp((char*)CAT_BUFF,"TX0;",4) == 0){
    if( frequency >= 10000000 )ubitx_mode = MODE_USB;
    else ubitx_mode = MODE_LSB;
    setFrequency(frequency);
    updateDisplay();
    TX_In_Progress = true;

    startTx(TX_SSB);
    
  }else if( strncmp((char*)CAT_BUFF,"VV;",3) == 0){
    if( vfoActive == VFO_A )vfoB = frequency;
    else {
      vfoB = vfoA;
      frequency = vfoA;
    }
    updateDisplay();
    
  }   

}

///////////////////////////////////////////////////////////////////////////////////////////////

void  Process_CAT_Command( void ){

  // wait for command
  while( Serial.available() > 0 ){
    CAT_BUFF[Cat_Buffer_Index] = Serial.read();

    if( CAT_BUFF[Cat_Buffer_Index ] == ';'){
      CAT_BUFF[Cat_Buffer_Index+1 ] = 0;
 		  Process_Command(); 
		  Cat_Buffer_Index = 0;
	  }else{
	    Cat_Buffer_Index = Cat_Buffer_Index +1;
	  }
  }
}




