// File name ubitx_factory_alignment.cpp

 
#include <Arduino.h>
#include <EEPROM.h>
#include "ubitx.h"
#include <LiquidCrystal_I2C.h>
extern LiquidCrystal_I2C lcd;


extern int  btnDown(void);
extern void printLine1(char *c);
extern void printLine2(char *c);
extern void si5351_set_calibration(int32_t cal);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern int  enc_read(void);
extern void startTx(byte txMode);
extern void stopTx(void);
extern void initOscillators(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void checkPTT(void);
extern void printLine(char linenmbr, char *c);
extern void ritDisable(void);


extern unsigned long usbCarrier;
extern byte menuOn;
extern bool swlUSB;
extern char c[21], b[21];
extern volatile bool keyDown;
extern unsigned long cwTimeout;
extern unsigned long cwTimeout;
extern unsigned long frequency;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern char ritOn;
extern char vfoActive;
extern int Freq_Underline[6];
extern unsigned int SI5351BX_XTALPF;              // 1:6pf  2:8pf  3:10pf
extern volatile unsigned long Ubitx_Voltage;
 float temp_value;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Converted_Ubitx_Voltage;
extern unsigned long Volt_Divider;

/**
 * This procedure is only for those who have a signal generator/transceiver tuned to exactly 7.150 and a dummy load 
 */

void   MenuSetPf( void ){
int s;
int LoopSem = 1;

  EEPROM.get(XTALPF, SI5351BX_XTALPF);
  if( (SI5351BX_XTALPF < 1)||(SI5351BX_XTALPF > 3)) SI5351BX_XTALPF = 2;
  printLine1((char*)"Push to set");
  if (SI5351BX_XTALPF == 1){
    printLine2((char*)"XTAL 6pf");
  }else if (SI5351BX_XTALPF == 2){
    printLine2((char*)"XTAL 8pf");
  }else if (SI5351BX_XTALPF == 3){
    printLine2((char*)"XTAL 10pf");
  }
  while ( LoopSem ){
    s = enc_read();
    if (s != 0 ){
     if (s > 1) SI5351BX_XTALPF = SI5351BX_XTALPF + 1;
     else if (s < -1) SI5351BX_XTALPF = SI5351BX_XTALPF - 1;

     if( SI5351BX_XTALPF < 1 ) SI5351BX_XTALPF = 1;
      if( SI5351BX_XTALPF > 3 ) SI5351BX_XTALPF = 3;

      if (SI5351BX_XTALPF == 1){
       printLine2((char*)"XTAL 6pf");
      }else if (SI5351BX_XTALPF == 2){
       printLine2((char*)"XTAL 8pf");
      }else if (SI5351BX_XTALPF == 3){
        printLine2((char*)"XTAL 10pf");
      }
      delay(500);
    }else{
      //keep clear of any previous button press
      if (btnDown())LoopSem = 0;
    }
  }
  EEPROM.put(XTALPF, SI5351BX_XTALPF);
  while( btnDown())delay(10);

  
}

void menuSidebandToggle( int btn ){

  
  //keep clear of any previous button press
  while (btnDown()) delay(100);
  delay(100);
   
  if (!btn){
    if (ubitx_mode == MODE_USB){
      printLine2((char*)"Currently USB");
      printLine1((char*)"Press to LSB?");
    }else{
      printLine2((char*)"Currently LSB");
      printLine1((char*)"Set to USB?");
    }
    return;
  }

  if (ubitx_mode == MODE_USB) ubitx_mode = MODE_LSB;
  else if (ubitx_mode == MODE_LSB) ubitx_mode = MODE_USB;
  delay(500);
}


 //this is used by the si5351 routines in the ubitx_5351 file
extern int32_t calibration;
extern uint32_t si5351bx_vcoa;

void Update_Freq(void){
memset(c, 0, sizeof(c));
memset(b, 0, sizeof(b));

  if( ubitx_mode == MODE_USB )strcpy(c, "USB");
  else if( ubitx_mode == MODE_LSB )strcpy(c, "LSB");
  else if( ubitx_mode == MODE_CW )strcpy(c, "CW ");
  else if( ubitx_mode == MODE_CWR )strcpy(c, "CWR");
  strcat(c,"    ");
  ultoa(frequency, b, DEC);
  //one mhz digit if less than 10 M, two digits if more
    if (strlen(b) < 7){
      strcat(&c[4], "00.");
      strncat(c, &b[0], 3);
     strcat(c, ".");
      strncat(c, &b[3], 2);    
    }else if (strlen(b) < 8){
      strcat(&c[7], "0");
      strncat(c, &b[0], 1);
      strcat(c, ".");
      strncat(c, &b[1], 3);    
      strcat(c, ".");
      strncat(c, &b[4], 2);
    }else {
      strncat(c, b, 2);
      strcat(c, ".");
      strncat(c, &b[2], 3);
      strcat(c, ".");
      strncat(c, &b[5], 2);    
    }


  printLine(1, c);
  lcd.setCursor(Freq_Underline[Freq_Index], 1);

}

void CaldoTuning(){
  int s;
  unsigned long prev_freq;

  s = enc_read();
  if (s){
    prev_freq = frequency;
    
    if (s > 4)
      frequency += Freq_Incr[Freq_Index];
    else if (s >  2)
      frequency += Freq_Incr[Freq_Index];
    else if (s >  0)
      frequency +=  Freq_Incr[Freq_Index];
    else if (s <  0)
      frequency -= Freq_Incr[Freq_Index];
    else if (s < -2)
      frequency -= Freq_Incr[Freq_Index];
    else if (s < -4)
      frequency -= Freq_Incr[Freq_Index];
      
    if (prev_freq < 10000000l && frequency >= 10000000l) ubitx_mode = MODE_USB;
    else if (prev_freq >= 10000000l && frequency < 10000000l) ubitx_mode = MODE_LSB;
   
    setFrequency(frequency);
    Update_Freq();
    }
}


void menuSetupTuning( int btn ){
int i;
bool looper = true;

  //keep clear of any previous button press
  while (btnDown())
    delay(100);
  delay(100);

  if (!btn){
    printLine2((char *)"Tune station?");
    printLine1((char *)" ");
    return;
  }
    printLine2((char *)"Encoder to Exit");
    Update_Freq();

while( looper ){
  //only if the button is pressed
  if (btnDown()){
    delay(100);
    if (btnDown()){
      // OK button is definitely pushed so now detect how long it is pressed.
      i=0;
      while ( btnDown() ){
        delay(10);
        i++;
        if (i > BUTTON_DELAY_2 )printLine2((char *)"Exit Tune");
      }
      if (i <= BUTTON_DELAY_2 ){
        Freq_Index = Freq_Index - 1;
        if( Freq_Index < 0 )Freq_Index = 5;
        Update_Freq();
      }else{
        printLine(1, (char*)"");
        while(btnDown())delay(50);
        delay(100);
        return;
      }
    }
   }
    CaldoTuning();
    delay(100);
}
 
}




int menuSetupCalibration(int btn){
  int knob = 0;
  int32_t prev_calibration;
   
  if (!btn){
    printLine2((char *)"Set Calibration?");
    printLine1((char *)" ");
    return 0;
  }

  printLine1((char *)"Set to Zero-beat,");
  printLine2((char *)"press PTT to save");
  delay(1000);
  
  prev_calibration = calibration;
  calibration = 0;
  si5351_set_calibration(calibration);
  setFrequency(frequency);    
  
  strcpy(b, "cal:");
  ltoa(calibration/8750, c, 10);
  strcat(b, c);
  printLine2(b);     

  while (digitalRead(PTT) == HIGH && !btnDown())
  {
    knob = enc_read();

    if (knob > 0){
      calibration += 8750;
      usbCarrier += 120;
    }
    else if (knob < 0){
      calibration -= 8750;
      usbCarrier -= 120;
    }
    else
      continue; //don't update the frequency or the display

    si5351_set_calibration(calibration);
    si5351bx_setfreq(0, usbCarrier);
    setFrequency(frequency);    

    strcpy(b, "cal:");
    ltoa(calibration/8750, c, 10);
    strcat(b, c);
    printLine2(b);     
  }
  
  //save the setting
  if (digitalRead(PTT) == LOW){
    printLine1((char *)"Calibration set!");
    printLine2((char *)"Set Carrier now");
    EEPROM.put(MASTER_CAL, calibration);
    delay(2000);
  }
  else
    calibration = prev_calibration;

  printLine2((char *)"");
  initOscillators();
  //si5351_set_calibration(calibration);
  setFrequency(frequency);    
  updateDisplay();
}


void printCarrierFreq(unsigned long freq){

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  ultoa(freq, b, DEC);
  
  strncat(c, b, 2);
  strcat(c, ".");
  strncat(c, &b[2], 3);
  strcat(c, ".");
  strncat(c, &b[5], 1);
  printLine2(c);    
}

void menuSetupCarrier(int btn){
  int knob = 0;
  unsigned long prevCarrier;
   
  if (!btn){
    printLine2((char *)"Set the BFO");
    printLine1((char *)" ");
    return;
  }

  prevCarrier = usbCarrier;
  printLine1((char *)"Tune to best Signal");  
  printLine2((char *)"PTT to confirm. ");
  delay(1000);

  usbCarrier = 11995000l;
  si5351bx_setfreq(0, usbCarrier);
  printCarrierFreq(usbCarrier);

  //disable all clock 1 and clock 2 
  while (digitalRead(PTT) == HIGH && !btnDown())
  {
    knob = enc_read();

    if (knob > 0)
      usbCarrier += 50;
    else if (knob < 0)
      usbCarrier -= 50;
    else
      continue; //don't update the frequency or the display
      
    si5351bx_setfreq(0, usbCarrier);
    printCarrierFreq(usbCarrier);
    
    delay(100);
  }

  //save the setting
  if (digitalRead(PTT) == LOW){
    printLine2((char *)"Carrier set!    ");
    EEPROM.put(USB_CAL, usbCarrier);
    delay(1000);
  }
  else 
    usbCarrier = prevCarrier;

  si5351bx_setfreq(0, usbCarrier);          
  setFrequency(frequency);    
  updateDisplay();
  printLine2((char *)"");
}

void btnWaitForClick(){
  while(!btnDown())
    delay(50);
  while(btnDown())
    delay(50);
 delay(50);
}


void MenuExit(int btn){
  
  if (!btn){
    printLine2((char*)"Exit ");
    printLine1((char *)" ");
    return;
  }
  printLine2((char*)"Power Cycle");
  printLine1((char*)"Radio");
  while(1)delay(1000);
}


void menuSetupVolts( int btn ){
char Voltage_Text[21];
char c[21];
int s,i=1;

  if (!btn){
    printLine2((char *)"Calibrate Volts");
    return;
  }
  while (btnDown())delay(10);
  EEPROM.get(VOLT_DIVIDE,Volt_Divider);
  if(  (Volt_Divider > 2000) || (Volt_Divider < 0))Volt_Divider = 982;
    printLine2((char *)"Match your VCC");
    printLine1((char *)"Encoder to Exit");
   delay(2000);
  while (!btnDown()){
    s = enc_read();
    if (s > 4)
      Volt_Divider--;
    else if (s >  2)
      Volt_Divider--;
    else if (s >  0)
      Volt_Divider--;
    else if (s <  0)
      Volt_Divider++;
    else if (s < -2)
      Volt_Divider++;
    else if (s < -4)
      Volt_Divider++;
    else if ( i == 1)i=0;
    else continue;
    itoa(Volt_Divider,Voltage_Text,10);
    sprintf(c,"Encoder  %d",Volt_Divider);
    printLine2(c);
    Ubitx_Voltage = analogRead(UBIT_VCC);
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)150);
    Ubitx_Voltage = analogRead(UBIT_VCC);
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)150);
    Converted_Ubitx_Voltage = Converted_Ubitx_Voltage  / Volt_Divider;
    ultoa(Converted_Ubitx_Voltage,b, 10);
    strcpy(c,"         ");
    if( strlen(b) == 1){
     strcat(c,"00");
      strcat(c,".");
      strncat(c,b,1);
    }else if( strlen(b) == 2){
      strcat(c,"0");
     strncat(c,b,1);
      strcat(c,".");
      strncat(c,&b[1],1);
    }else{
     strncat(c,b,2);
      strcat(c,".");
     strncat(c,&b[2],1);
    }
    strcat(c,"V");
    printLine(1,c);
    lcd.setCursor(9,0);
  }
  while (btnDown())delay(10);
    printLine1((char*)"");
    printLine2((char*)"");
  EEPROM.put(VOLT_DIVIDE,Volt_Divider);
}


void doAlignment(void){
  int select=0, i;
  bool btnState;

  
 printLine2((char*)"ALIGNMENT");
 printLine1((char*)" ");
 delay(100);

 while ( btnDown() )delay(100);

 lcd.clear();
  //wait for the button to be raised up
  while(btnDown())
    delay(50);
  delay(50);  //debounce


  ubitx_mode = MODE_LSB;
  ritDisable();

  // must set Xtal pf
  MenuSetPf();
  menuOn = 2;

  
  while (menuOn){
    i = enc_read();
    btnState = btnDown();

    if (i > 1){
      if (select + i < 50)
        select += i;
    }
    if (i < 0 && select - i >= 0)
      select += i;      //caught ya, i is already -ve here, so you add it

    if (select < 10)
      MenuExit(btnState);
    else if (select < 20)
     menuSetupTuning(btnState);
    else if (select < 30)
     menuSetupCalibration(btnState);   //crystal
    else if (select < 40)
      menuSetupCarrier(btnState);       //lsb
    else if (select < 50)
      menuSetupVolts(btnState);       //lsb
  }
  //debounce the button
  while(btnDown())
    delay(50);
  delay(50);
}


