/** Menus
 *  File Name ubitx_menu.cpp
 *  The Radio menus are accessed by tapping on the function button. 
 *  - The main loop() constantly looks for a button press and calls doMenu() when it detects
 *  a function button press. 
 *  - As the encoder is rotated, at every 10th pulse, the next or the previous menu
 *  item is displayed. Each menu item is controlled by it's own function.
 *  - Each menu function may be called to display itself
 *  - Each of these menu routines is called with a button parameter. 
 *  - The btn flag denotes if the menu item was clicked on or not.
 *  - If the menu item is clicked on, then it is selected,
 *  - If the menu item is NOT clicked on, then the menu's prompt is to be displayed
 */
#include <Arduino.h>
#include <EEPROM.h>
#include "ubitx.h"

#include <LiquidCrystal_I2C.h>
extern LiquidCrystal_I2C lcd;//  indicate that "lcd" was already defined in <ubitx_V1_08_I2C>

extern void stopTx(void);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern bool btnDown(void);
extern void ritDisable(void);
extern void ritEnable(void);
extern int  enc_read(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void printLine1(char *c);
extern void si5351_set_calibration(int32_t cal);
extern void startTx(byte txMode);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern void initOscillators(void);
extern int Test_Button( char * title1, char * title2,char * title3,char * title4 );



extern unsigned long frequency;
extern char ritOn;
extern char splitOn;
extern byte menuOn;
extern char vfoActive;
extern unsigned long vfoA;
extern unsigned long vfoB;
extern boolean modeCalibrate;
extern int cwSpeed;
extern char c[21], b[21];
extern char keyDown;
extern unsigned long cwTimeout;
extern unsigned long usbCarrier;
extern unsigned long sideTone;
extern char Version_Text[16];
extern char Version_Date[16];
extern int Menu_Select_Index[7][2];         // new menu
extern int Menu_Index;                      // new menu
extern volatile int ubitx_mode;
extern bool Volts_Active;
extern int Vfo_Mode[2];
extern unsigned volatile char IAMBIC;  // 0 for Iambic A, 0x10 for Iambic B
extern unsigned volatile char PDLSWAP;     // 0x00 for normal, 0x08 for swap
extern volatile unsigned char keyerControl;
extern unsigned volatile char IAMBIC;



char ubitx_mode_text[6][5] = { "USB","LSB","CW ","CWr","SWU","SWL" };

char ubitx_CAT_Baud_rates[8][7] = {"1200","2400","4800","9600","19200","38400","57600","115200"};
long ubitx_CAT_Long_Baud_rates[8] = {1200,2400,4800,9600,19200,38400,57600,115200};

int wpm;

int Menu_Cursor_Line_Postions[7] =  {0,0,0,0,0,0,1};
int Menu_Cursor_Index_Postions[7] = {0,3,4,6,11,15,4};
int Max_Menus_Items = 6;
int Current_Menu = 0;
int Last_Menu = 0;
int Last_Mode = 0;
int ubitx_baud = 5;
int Last_Baud = 5;
long  CW_TIMEOUT = 350l;
bool Volts_Active = false;
////////////////////////////////////////////////////////////

void DoBaud( void ){

int BaudmenuOn = 2;
int i,btnState;
    
  ubitx_baud = Last_Baud;
  printLine2(ubitx_CAT_Baud_rates[ubitx_baud]);
  //wait for the button menu select button to be lifted)
  while (btnDown()) delay(50);
  delay(50);    
  
  printLine2(ubitx_CAT_Baud_rates[ubitx_baud]);
  lcd.setCursor(0,2 );
  
  while (BaudmenuOn){
    
    i = enc_read();
    if( i != 0){
      if (i > 1){
        ubitx_baud++;
        if( ubitx_baud > 7) ubitx_baud = 7;
      }else if (i < -1 ){
        ubitx_baud--;
        if( ubitx_baud < 0) ubitx_baud = 0;
      }
      printLine2(ubitx_CAT_Baud_rates[ubitx_baud]);
      lcd.setCursor(0,0 );
      delay(500);
    }
    if( btnDown() ){
      while ( btnDown() ) delay(50);
      BaudmenuOn = 0;
    }
 }
    Last_Baud = ubitx_baud;
    BaudmenuOn = ubitx_baud;
    EEPROM.put(BAUD, BaudmenuOn );
    printLine2((char *)"Must Pwr Cycle");
    lcd.setCursor(0,0 );
    delay(4000);
    updateDisplay();
    menuOn = 0;

  
}

////////////////////////////////////////////////////////////


void SetIambicA( void ){
int temp;

  IAMBIC = 0;
  keyerControl = keyerControl & 0xEF;
  temp = IAMBIC;
  EEPROM.put(IAMBIC_MODE, IAMBIC );
  delay(200);
  updateDisplay();
}

void SetIambicB( void ){
int temp;
  
  IAMBIC =  0x10;
  keyerControl = keyerControl | IAMBIC;
  temp = IAMBIC;
  EEPROM.put(IAMBIC_MODE, IAMBIC );
  delay(200);
  updateDisplay();
}

void menuIambicToggle( void ){
int temp;

  if ( IAMBIC == 0 )SetIambicB();
  else SetIambicA();
  menuOn = 0;
  
}

void SetPaddleSwapN( void ){

    PDLSWAP = 0;
    keyerControl = keyerControl & 0xF7;

}

void SetPaddleSwapR( void ){
  
    PDLSWAP = 0x08;
    keyerControl = keyerControl | PDLSWAP;
  
}

  

void menuPaddleSwapToggle( void ){
int temp;

  if ( PDLSWAP == 0 ) SetPaddleSwapR();
  else  SetPaddleSwapN();
  temp = PDLSWAP;
  EEPROM.put(PDL_SWAP_VAL, temp );
  delay(200);
  updateDisplay();
  menuOn = 0;

  
}
  

void menuVfoToggle( void ){
  
  if(splitOn){
      if (vfoActive == VFO_B)vfoActive = VFO_A;
      else vfoActive = VFO_B;
  }else{
      if (vfoActive == VFO_B){
        vfoB = frequency;
        EEPROM.put(VFO_B, frequency);
        vfoActive = VFO_A;
        frequency = vfoA;
      }else {
        vfoA = frequency;
        EEPROM.put(VFO_A, frequency);
        vfoActive = VFO_B;
        frequency = vfoB;
      }
  }   
      ritDisable();
      setFrequency(frequency);
      updateDisplay();
      delay(200);
      //exit the menu
      menuOn = 0;
  
}

void menuRitToggle( void ){
      if (ritOn == 0){
        ritEnable();
      }else{
        ritDisable();
      }
      menuOn = 0;
      delay(100);
      updateDisplay();
 }

void menuModeSelect( void ){
int modemenuOn = 2;
int i,btnState;
int temp;
    
  temp = Last_Mode;
  printLine2(ubitx_mode_text[ubitx_mode]);
  lcd.setCursor(0,0 );
  //wait for the button menu select button to be lifted)
  while (btnDown()) delay(50);
  delay(50);    
  ritDisable();
  
  
  while (modemenuOn){  
    i = enc_read();
    if( i != 0){
      if (i > 1){
        temp++;
        if( temp > 5) temp = 5;
      }else if (i < -1 ){
        temp--;
        if( temp < 0) temp = 0;
      }
      printLine2(ubitx_mode_text[temp]);
      lcd.setCursor(0,0 );
      delay(500);
    }
    if( btnDown() ){
      while ( btnDown() ) delay(50);
      modemenuOn = 0;
    }
 }
      Last_Mode = temp;
    ubitx_mode = Last_Mode;
    menuOn = ubitx_mode;
    EEPROM.put(MODE, menuOn );
    delay(200);
    setFrequency(frequency);
    updateDisplay();
    menuOn = 0;
}



void menuSplitToggle( void ){

  if( splitOn ){
    splitOn = 0;
  }else{
    splitOn = 1;
    vfoB = vfoA;
  }
  updateDisplay(); 
}




void menuCWSpeed( void ){
    int knob = 0;


    strcpy(b, "           S");
    itoa(wpm,c, 10);
    strcat(b, c);
    printLine2(b);
    delay(200);
    lcd.setCursor(13,0 );
    while(!btnDown() && digitalRead(PTT) == HIGH){

      knob = enc_read();
      if (knob != 0){
        if (wpm > 5 && knob < 0)
          wpm--;
        if (wpm < 50 && knob > 0)
          wpm++;

    strcpy(b, "           S");
        itoa(wpm,c, 10);
        strcat(b, c);
        printLine2(b);
    lcd.setCursor(13,0 );
      }
      //abort if this button is down
      if (btnDown())
        //re-enable the clock1 and clock 2
        break;
    }
    
    //save the setting
    cwSpeed = 600 / wpm;
    EEPROM.put(CW_SPEED, wpm);
    delay(200);
    menuOn = 0;
    updateDisplay();
}


void menuCWDelay( void ){
    int knob = 0;


    strcpy(b, "              D");
    itoa(CW_TIMEOUT,c, 10);
    strcat(b, c);
    printLine2(b);
    delay(200);
    lcd.setCursor(17,0 );
    while(!btnDown() && digitalRead(PTT) == HIGH){

      knob = enc_read();
      if (knob != 0){
        if (CW_TIMEOUT > TX_DELAY_MIN && knob < 0)
          CW_TIMEOUT--;
        if (CW_TIMEOUT < TX_DELAY_MAX && knob > 0)
          CW_TIMEOUT++;

    strcpy(b, "              D");
        itoa(CW_TIMEOUT,c, 10);
        strcat(b, c);
        printLine2(b);
    lcd.setCursor(17,0 );
      }
      //abort if this button is down
      if (btnDown())
        //re-enable the clock1 and clock 2
        break;
    }
    
    //save the setting
    EEPROM.put(TX_DELAY, CW_TIMEOUT);
    delay(200);
    menuOn = 0;
    updateDisplay();
}



void menuSetupCwTone( void ){
    int knob = 0;
     

    strcpy(c,"      T");
    itoa(sideTone, b, 10);
    strcat(c,b);
    printLine2(c);
    delay(200);
    tone(CW_TONE, sideTone);
    lcd.setCursor(8,0 );

    //disable all clock 1 and clock 2 
    while (digitalRead(PTT) == LOW || !btnDown())
    {
      knob = enc_read();

      if (knob > 0 && sideTone < 990)
        sideTone += 10;
      else if (knob < 0 && sideTone > 100 )
        sideTone -= 10;
      else
        continue; //don't update the frequency or the display
        
      tone(CW_TONE, sideTone);
      strcpy(c,"      T");
      itoa(sideTone, b, 10);
      strcat(c,b);
      printLine2(c);
      lcd.setCursor(8,0 );

      delay(100);
    }
    noTone(CW_TONE);
    //save the setting
    EEPROM.put(CW_SIDETONE, sideTone);
    delay(200);
    updateDisplay(); 
    menuOn = 0; 
 }

void Process_Volts( void ){
  
    if (Volts_Active){
      Volts_Active = false;
    }else{
      Volts_Active = true;
    }
    menuOn = Volts_Active;
    EEPROM.put(VOLTS, menuOn );
    delay(200);
    updateDisplay();
    menuOn = 0;

  
}


void doMenu(void){
  int i;
  int Do_Something;

  Current_Menu = Last_Menu;
  updateDisplay();
  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu]);
 
  //wait for the button to be raised up
  while(btnDown()) delay(50);
  delay(50);  //debounce
  
  menuOn = 2;

  while (menuOn){
    Last_Menu = Current_Menu;
  
    // check if user moving to different menu selection    
    i = enc_read();
    if( i != 0){
      if( (ubitx_mode == MODE_USB)||(ubitx_mode == MODE_LSB)||(ubitx_mode == MODE_SWU)||(ubitx_mode == MODE_SWL)){
        if (i > 1){
          if( Current_Menu == 0)Current_Menu = Max_Menus_Items;
          else Current_Menu = Current_Menu + 1;
          if (Current_Menu > Max_Menus_Items ) Current_Menu = 0;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
        }else if (i < -1 ){
          if( Current_Menu == 0)Current_Menu = Max_Menus_Items;
          else if( Current_Menu == 5)Current_Menu = 0;
          else Current_Menu = Current_Menu - 1;
          if (Current_Menu < 0 ) Current_Menu = Max_Menus_Items;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
        }
      }else if( (ubitx_mode == MODE_CW)||(ubitx_mode == MODE_CWR)){
        if (i > 1){
          if( (splitOn) && (Current_Menu == 0))Current_Menu = 4;
          else if( (ritOn) && (Current_Menu == 0))Current_Menu = 4;
          else Current_Menu = Current_Menu + 1;
          if (Current_Menu > Max_Menus_Items ) Current_Menu = 0;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
        }else if (i < -1 ){
          if( (splitOn) && (Current_Menu == 4))Current_Menu = 0;
          else if( (ritOn) && (Current_Menu == 4))Current_Menu = 0;
          else Current_Menu = Current_Menu - 1;
          if (Current_Menu < 0 ) Current_Menu = Max_Menus_Items;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
        }
      }
      delay(500);
    }else{
    
  Do_Something = Test_Button((char*)"Change",(char*)"Exit Menu",(char*)"Exit Menu",(char*)"Exit Menu");
  if( Do_Something > 1  ) menuOn = 0;
  else if( Do_Something == 1 ){

    switch  ( Current_Menu ) {
      
      case  (0):
        menuModeSelect();
        break;
      case  (1):
        menuPaddleSwapToggle();
        break;
      case  (2):
        menuIambicToggle();
        break;
      case  (3):
        menuSetupCwTone();
        break;
      case  (4):
        menuCWSpeed();
        break;
      case  (5):
        menuCWDelay();
        break;
      case  (6):
        Process_Volts();
        break;
    }
  }
  //debounce the button
  while(btnDown()) delay(50);
  delay(50);
  
    }}
      updateDisplay();

    }

