/**
 * ubitx_ui.cpp
 * The user interface of the uBITX consists of the encoder, the push-button on top of it
 * and the 16x2 LCD display.
 * The upper line of the display is constantly used to display frequency and status
 * of the radio. Occasionally, it is used to provide a two-line information that is 
 * quickly cleared up.
 */

//      
#include <Arduino.h>
#include "ubitx.h"
#include <LiquidCrystal_I2C.h>
extern LiquidCrystal_I2C lcd;//  indicate that "lcd" was already defined in <ubitx_V1_08_I2C>



extern void printLine1(char *c);
extern char printBuff[2][17];
extern char c[21], b[21];
extern unsigned long frequency;
extern unsigned long vfoA, vfoB;
extern unsigned long sideTone;    // new menu
extern char inTx;
extern char ritOn;
extern char splitOn;
extern bool isUSB;
extern char vfoActive;
extern int Freq_Underline[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern int cwSpeed;
extern long CW_TIMEOUT;
extern int wpm;
extern long ritOffset;
extern bool Volts_Active;
extern volatile bool Ubitx_Voltage_Act;
extern unsigned volatile char IAMBIC;
extern unsigned volatile char PDLSWAP;


/////////////////////////////////////////////////////////////////////////////
volatile unsigned long Ubitx_Voltage = 0;
volatile unsigned long Current_Ubitx_Voltage = 0;
volatile unsigned long Converted_Ubitx_Voltage = 0;
extern unsigned long Volt_Divider;

volatile int Ubitx_Voltage_Timer = 0;
char Voltage_Text[7];

//returns true if the button is pressed
bool btnDown(void){
  
  if (digitalRead(FBUTTON) == HIGH) return false;
  else return true;
}


// The generic routine to display one line on the LCD 
void printLine(char linenmbr, char *c) {
int i;

   uint8_t oldSREG = SREG;
   sei();
  
  if (strcmp(c, printBuff[linenmbr])) {     // only refresh the display when there was a change
    lcd.setCursor(0, linenmbr);             // place the cursor at the beginning of the selected line
    lcd.print(c);
    strcpy(printBuff[linenmbr], c);

    for (byte i = strlen(c); i < 20; i++) { // add white spaces until the end of the 16 characters line is reached
      lcd.print(' ');
    }
  }
  SREG=oldSREG;
}

//  short cut to print to the first line
void printLine3(char *c){
  printLine(2,c);
}

//  short cut to print to the first line
void printLine4(char *c){
  printLine(3,c);
}

//  short cut to print to the first line
void printLine1(char *c){
  printLine(1,c);
}
//  short cut to print to the first line
void printLine2(char *c){
  printLine(0,c);
}

// this builds up the top line of the display with frequency and mode
void updateDisplay(void) {
  //char local_buf[12];
  
  unsigned long temp;
  // tks Jack Purdum W8TEE
  // replaced fsprint commands by str commands for code size reduction

  //now, the second line
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

 
  if( ubitx_mode == MODE_USB )strcpy(c, "USB");
  else if( ubitx_mode == MODE_LSB )strcpy(c, "LSB");
  else if( ubitx_mode == MODE_CW )strcpy(c, "CW ");
  else if( ubitx_mode == MODE_CWR )strcpy(c, "CWr");
  else if( ubitx_mode == MODE_SWU )strcpy(c, "SWU");
  else if( ubitx_mode == MODE_SWL )strcpy(c, "SWL");
  if( ( ubitx_mode == MODE_CW )||( ubitx_mode == MODE_CWR )){
    if( PDLSWAP == 0 )strcat(c,"N");
    else strcat(c,"R");
    if( IAMBIC == 0 )strcat(c,"A");
    else strcat(c,"B");
    if( (!ritOn)&&(!splitOn)){
      strcat(c, " T");
      itoa(sideTone,b,10);
      strcat(c,b);
      strcat(c," S");
      itoa(wpm,b,10);
      strcat(c,b);
      strcat(c," D");
      itoa(CW_TIMEOUT,b,10);
      strcat(c,b);
    }
  }else strcat(c,"  ");
  if(splitOn){
    if( vfoActive == VFO_B )strcat(c," T");
    else strcat(c," t");
    ultoa(vfoB, b, DEC);
    if (strlen(b) < 7){
      strcat(&c[7], "00.");
      strncat(c, &b[0], 3);
     strcat(c, ".");
      strncat(c, &b[3], 2);    
    }else if (strlen(b) < 8){
      strcat(&c[7], "0");
      strncat(c, &b[0], 1);
      strcat(c, ".");
      strncat(c, &b[1], 3);    
      strcat(c, ".");
      strncat(c, &b[4], 2);
    }else {
      strncat(c, b, 2);
      strcat(c, ".");
      strncat(c, &b[2], 3);
      strcat(c, ".");
      strncat(c, &b[5], 2);    
    }
  }else if (ritOn){
    strcat(c, " Rit ");
    if( ritOffset < 0 ){
      strcat(c, "-");
      temp = 0 - ritOffset;
    }else{
      strcat(c, "+");
      temp = ritOffset;
    }
    ultoa(temp/100, b, DEC);
    if( strlen(b) == 1){
      strcat(c,"00.");
      strncat(c,&b[0],1);
    }
    if( strlen(b) == 2){
      strcat(c,"0");
      strncat(c,&b[0],1);
      strcat(c,".");
      strncat(c,&b[1],1);
    }
    if( strlen(b) == 3){
      strncat(c,&b[0],2);
      strcat(c,".");
      strncat(c,&b[2],1);
    }
  }
  printLine2(c);

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  // Display VCC volts 
  if( Volts_Active ){ 
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)150);
    Converted_Ubitx_Voltage = Converted_Ubitx_Voltage  / Volt_Divider;
    ultoa(Converted_Ubitx_Voltage,b, 10);
    if( strlen(b) == 1){
     strcpy(c,"00");
      strcat(c,".");
      strncat(c,b,1);
    }else if( strlen(b) == 2){
      strcpy(c,"0");
     strncat(c,b,1);
      strcat(c,".");
      strncat(c,&b[1],1);
    }else{
     strncpy(c,b,2);
      strcat(c,".");
     strncat(c,&b[2],1);
    }
    strcat(c,"V ");
    strcpy(Voltage_Text,c);
    Current_Ubitx_Voltage = Ubitx_Voltage;
  }else strcpy(c,"    V ");
  
  ultoa(vfoA, b, DEC);
  if ( splitOn){
    if( vfoActive == VFO_A )strcat(c,"R");
    else strcat(c,"r");
  }
  else if(vfoActive == VFO_A )strcat(c, "A");
  
  else{
   ultoa(vfoB, b, DEC);
   strcat(c, "B");
  }
  if (strlen(b) < 7){
    strcat(&c[7], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);    
  }else if (strlen(b) < 8){
    strcat(&c[7], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);    
    strcat(c, ".");
    strncat(c, &b[4], 2);
  }else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);    
  }


  printLine(1, c);
  if (!ritOn){
//    uint8_t oldSREG = SREG;
//   sei();
    if((splitOn)&&(vfoActive == VFO_B))lcd.setCursor(Freq_Underline[Freq_Index], 0);
    else lcd.setCursor(Freq_Underline[Freq_Index], 1);
//  SREG=oldSREG;
  }else{
//    uint8_t oldSREG = SREG;
//    sei();
    lcd.setCursor(15,2);
//    SREG=oldSREG;
  }
 
}

int enc_prev_state = 3;

/**
 * The A0 And A1 are purely analog lines on the Arduino Nano
 * These need to be pulled up externally using two 10K resistors
 * 
 * There are excellent pages on the Internet about how these encoders work
 * and how they should be used. We have elected to use the simplest way
 * to use these encoders without the complexity of interrupts etc to 
 * keep it understandable.
 * 
 * The enc_state returns a two-bit number such that each bit reflects the current
 * value of each of the two phases of the encoder
 * 
 * The enc_read returns the number of net pulses counted over 50 msecs. 
 * If the puluses are -ve, they were anti-clockwise, if they are +ve, the
 * were in the clockwise directions. Higher the pulses, greater the speed
 * at which the enccoder was spun
 */

byte enc_state (void) {
    return (analogRead(ENC_A) > 500 ? 1 : 0) + (analogRead(ENC_B) > 500 ? 2: 0);
}

int enc_read(void) {
  int result = 0; 
  byte newState;
  int enc_speed = 0;
  
  long stop_by = millis() + 50;
  
  while (millis() < stop_by) { // check if the previous state was stable
    newState = enc_state(); // Get current state  
    
    if (newState != enc_prev_state)
      delay (1);
    
    if (enc_state() != newState || newState == enc_prev_state)
      continue; 
    //these transitions point to the encoder being rotated anti-clockwise
    if ((enc_prev_state == 0 && newState == 2) || 
      (enc_prev_state == 2 && newState == 3) || 
      (enc_prev_state == 3 && newState == 1) || 
      (enc_prev_state == 1 && newState == 0)){
        //result--;
        result++;
      }
    //these transitions point o the enccoder being rotated clockwise
    if ((enc_prev_state == 0 && newState == 1) || 
      (enc_prev_state == 1 && newState == 3) || 
      (enc_prev_state == 3 && newState == 2) || 
      (enc_prev_state == 2 && newState == 0)){
        //result++;
        result--;
      }
    enc_prev_state = newState; // Record state for next pulse interpretation
    enc_speed++;
    delay(1);
  }


  return(result);
}


